@@ -2,10 +2,13 @@ import asyncio
 import functools
 import inspect
 import traceback
-from typing import Callable, Any
+from typing import Awaitable, Callable, ParamSpec, TypeVar
 
 from tp_helper import get_full_class_name
 
+P = ParamSpec("P")  # параметры оригинальной функции
+R = TypeVar("R")  # возвращаемый тип оригинальной функции
+
 
 def retry_forever(
     start_message: str,
@@ -14,24 +17,36 @@ def retry_forever(
     backoff: float = 1.2,
     max_delay: int = 60,
     discord_every: int = 3,
-):
-    def decorator(func: Callable) -> Callable:
+    ignore_exceptions: list[type[Exception | BaseException]] | None = None,
+) -> Callable[
+    [Callable[P, Awaitable[R]]], Callable[P, Awaitable[R]]
+]:  # принимает и возвращает асинхронную функцию с исходной сигнатурой
+    """
+    Оборачивает только метод класса.
+    """
+    if ignore_exceptions is None:
+        ignore_exceptions = [],
+
+    def decorator(func: Callable[P, Awaitable[R]]) -> Callable[P, Awaitable[R]]:
         sig = inspect.signature(func)
 
         @functools.wraps(func)
-        async def wrapper(self, *args: Any, **kwargs: Any) -> Any:
+        async def wrapper(*args: P.args, **kwargs: P.kwargs) -> R:
             # --- Собираем контекст для подстановки в сообщения
+            self = args[0] if args else None  # self итак уже есть в *args
+            if self is None:
+                raise ValueError(
+                    "@retry_forever применяется только к методу класса, "
+                    f"но {func.__qualname__} не принимает аргументов"
+                )
             try:
-                bound = sig.bind(self, *args, **kwargs)
+                bound = sig.bind(*args, **kwargs)
                 bound.apply_defaults()
                 context = dict(bound.arguments)
             except Exception:
                 context = {"self": self}
 
-            str_context = {}
-
-            for k, v in context.items():
-                str_context[k] = str(v)
+            str_context = {k: str(v) for k, v in context.items()}
 
             # --- Распаковываем self.*
             if "self" in context:
@@ -57,8 +72,10 @@ def retry_forever(
 
             while True:
                 try:
-                    return await func(self, *args, **kwargs)
+                    return await func(*args, **kwargs)
                 except Exception as e:
+                    if type(e) in ignore_exceptions:
+                        raise e from e
                     retry_count += 1
 
                     str_context_with_exception = {
@@ -85,9 +102,7 @@ def retry_forever(
                     if (
                         retry_count % discord_every == 0
                         and hasattr(self, "discord")
-                        and callable(
-                            getattr(self.discord, "send_traceback_report", None)
-                        )
+                        and callable(getattr(self.discord, "send_traceback_report", None))
                     ):
                         try:
                             await self.discord.send_traceback_report(e, err_msg)
