@@ -1,5 +1,8 @@
 import inspect
+import json
 import traceback
+from datetime import datetime, UTC
+from io import BytesIO
 from pathlib import Path
 
 import aiohttp
@@ -14,6 +17,7 @@ class DiscordHelper:
     MAX_DISCORD_CONTENT = 2000
     MAX_DISCORD_EMBED_DESC = 4096
     MAX_DISCORD_EMBED_TITLE = 256
+    MAX_DISCORD_FILE_SIZE = 8 * 1024 * 1024
 
     def __init__(self, url: str):
         self.url: str = url
@@ -21,7 +25,8 @@ class DiscordHelper:
         self.description: Optional[str] = None
         self.color: Optional[int] = None
         self.notify_everyone: bool = False
-        self.proxy: Optional[str] = None  # Прокси-сервер (по умолчанию отключен)
+        self.proxy: Optional[str] = None
+        self.files = []
 
     def reset(self) -> "DiscordHelper":
         """Сбрасывает все параметры сообщения к значениям по умолчанию."""
@@ -29,6 +34,7 @@ class DiscordHelper:
         self.description = None
         self.color = None
         self.notify_everyone = False
+        self.files = []
         return self
 
     def set_proxy(self, proxy_url: str) -> "DiscordHelper":
@@ -57,6 +63,25 @@ class DiscordHelper:
         self.description = description
         return self
 
+    def add_file_from_str(
+        self, filename: str, content: str, encoding: str = "utf-8"
+    ) -> "DiscordHelper":
+        """
+        Добавляет файл, созданный из строки (например, лог, CSV и т.д.)
+        """
+
+        file_bytes = content.encode(encoding)
+
+        if len(file_bytes) > self.MAX_DISCORD_FILE_SIZE:
+            truncated_notice = b"\n\n--- [truncated] ---"
+            max_bytes = self.MAX_DISCORD_FILE_SIZE - len(truncated_notice) - 10
+            file_bytes = file_bytes[:max_bytes].rstrip() + truncated_notice
+            print(f"⚠️ Файл '{filename}' был обрезан до 8MB для отправки в Discord")
+
+        file_obj = BytesIO(file_bytes)
+        self.files.append((filename, file_obj))
+        return self
+
     def set_color(self, color: int) -> "DiscordHelper":
         """Устанавливает цвет сообщения (в формате int)."""
         self.color = color
@@ -95,26 +120,33 @@ class DiscordHelper:
         self.set_notify_everyone()
         await self.send_with_level("Error", message, desc)
 
-    async def send_traceback_report(self, e: Exception, desc: str) -> None:
+    async def send_traceback_report(
+        self,
+        e: Exception,
+        message: str = None,
+        desc: str = None,
+    ) -> None:
         """
         Отправляет сообщение об ошибке с автоматически сформированным заголовком вида:
         (ClassName) file_name.py
+        И прикладывает traceback как файл.
         """
+
+        if message is None:
+            message = f"\n{type(e).__name__}: {str(e)}"
+
         tb = traceback.extract_tb(e.__traceback__)
         if tb:
             last = tb[-1]
             filename = Path(last.filename).name
             lineno = last.lineno
-            line = last.line
         else:
             filename = "unknown.py"
             lineno = -1
-            line = ""
 
-            # Пытаемся найти имя класса из стека вызова
+        # Пытаемся найти имя класса из стека вызова
         class_name = "UnknownClass"
-        stack = inspect.stack()
-        for frame_info in stack:
+        for frame_info in inspect.stack():
             self_obj = frame_info.frame.f_locals.get("self")
             if self_obj and self_obj.__class__.__name__ != self.__class__.__name__:
                 class_name = self_obj.__class__.__name__
@@ -128,9 +160,14 @@ class DiscordHelper:
         tb_text = "".join(
             traceback.format_exception(type(e), e, e.__traceback__)
         ).strip()
-        formatted_message = f"```{tb_text}```"
 
-        await self.send_error(message=formatted_message, desc=desc)
+        # Прикрепляем traceback как файл
+        utc_now = datetime.now(UTC).strftime("%Y-%m-%d_%H-%M-%S")
+        file_name = f"traceback_{utc_now}.log"
+
+        self.add_file_from_str(file_name, tb_text)
+
+        await self.send_error(message=message, desc=desc)
 
     async def send_warning(self, message: str = None, desc: Optional[str] = None):
         """Отправляет предупреждающее сообщение."""
@@ -149,8 +186,6 @@ class DiscordHelper:
         await self._send_message(message)
 
     async def _send_message(self, content: str):
-        """Отправляет сообщение в Discord через Webhook с использованием прокси (если задано)."""
-
         content = f"{'@everyone ' if self.notify_everyone else ''}{content}"
         content = self._trim(content, self.MAX_DISCORD_CONTENT)
 
@@ -172,13 +207,33 @@ class DiscordHelper:
 
         try:
             async with aiohttp.ClientSession() as session:
-                async with session.post(
-                    self.url, json=payload, proxy=self.proxy
-                ) as response:
-                    if response.status != 204:
-                        print(
-                            f"Ошибка отправки в Discord: {response.status} - {await response.text()}"
+                if self.files:
+                    form = aiohttp.FormData()
+                    form.add_field("payload_json", json.dumps(payload))
+
+                    for idx, (filename, file_obj) in enumerate(self.files):
+                        form.add_field(
+                            f"file{idx}",
+                            file_obj,
+                            filename=filename,
+                            content_type="application/octet-stream",
                         )
+
+                    async with session.post(
+                        self.url, data=form, proxy=self.proxy
+                    ) as response:
+                        if response.status not in (200, 204):
+                            print(
+                                f"Ошибка отправки в Discord: {response.status} - {await response.text()}"
+                            )
+                else:
+                    async with session.post(
+                        self.url, json=payload, proxy=self.proxy
+                    ) as response:
+                        if response.status != 204:
+                            print(
+                                f"Ошибка отправки в Discord: {response.status} - {await response.text()}"
+                            )
         except Exception as e:
             print(traceback.format_exc())
             print(f"Ошибка при отправке сообщения в Discord: {e}")
